@@ -6,7 +6,7 @@
 ;; Maintainer: Protesilaos Stavrou General Issues <~protesilaos/general-issues@lists.sr.ht>
 ;; URL: https://git.sr.ht/~protesilaos/spacious-padding
 ;; Mailing-List: https://lists.sr.ht/~protesilaos/general-issues
-;; Version: 0.1.0
+;; Version: 0.2.0
 ;; Package-Requires: ((emacs "28.1"))
 ;; Keywords: convenience, focus, writing, presentation
 
@@ -49,33 +49,115 @@
 
 (defgroup spacious-padding ()
   "Increase the padding/spacing of frames and windows."
+  :group 'faces
   :group 'frames)
 
 (defcustom spacious-padding-widths
   '( :internal-border-width 15
+     :header-line-width 4
+     :mode-line-width 6
+     :tab-width 4
      :right-divider-width 30
      :scroll-bar-width 8)
-  "Number of pixels for frame and window divider border width."
+  "Set the pixel width of individual User Interface elements.
+This is a plist of the form (:key1 value1 :key2 value2).  The
+value is always a natural number.  Keys are keywords among the
+following:
+
+- `:internal-border-width' refers to the space between the
+  boundaries of the Emacs frame and where the text contents
+  start.
+
+- `:right-divider-width' is the space between two side-by-side
+  windows.
+
+- `:tab-width' concerns the padding around tab buttons.  For the
+  time being, `tab-bar-mode' is the only one affected.
+
+- `:header-line-width', `mode-line-width', `scroll-bar-width'
+  point to the header-line, mode-line, and scroll-bar,
+  respectively."
   :type '(plist
           :key-type (choice (const :internal-border-width)
                             (const :right-divider-width)
+                            (const :tab-width)
+                            (const :header-line-width)
+                            (const :mode-line-width)
                             (const :scroll-bar-width))
           :value-type natnum)
+  :package-version '(spacious-padding . "0.2.0")
   :group 'spacious-padding)
 
+(defvar spacious-padding--mode-line-faces
+  '(mode-line mode-line-active mode-line-inactive mode-line-highlight)
+  "Mode line faces relevant to `spacious-padding-mode'.")
+
+(defvar spacious-padding--header-line-faces
+  '(header-line header-line-highlight)
+  "Header line faces relevant to `spacious-padding-mode'.")
+
+;; TODO 2023-11-16: Cover `tab-line-mode'.
+(defvar spacious-padding--tab-faces
+  '(tab-bar tab-bar-tab tab-bar-tab-inactive)
+  "Tab faces relevant to `spacious-padding-mode'.")
+
+(defun spacious-padding--get-box-width (key)
+  "Get width for :box of face represented by KEY in `spacious-padding-widths'.
+Return 4 if KEY does not have a value."
+  (or (plist-get spacious-padding-widths key) 4))
+
+(defun spacious-padding--get-face-width (face)
+  "Return width of FACE from `spacious-padding-widths'."
+  (cond
+   ((memq face spacious-padding--mode-line-faces)
+    (spacious-padding--get-box-width :mode-line-width))
+   ((memq face spacious-padding--header-line-faces)
+    (spacious-padding--get-box-width :header-line-width))
+   ((memq face spacious-padding--tab-faces)
+    (spacious-padding--get-box-width :tab-width))
+   (t (error "`%s' is not relevant to `spacious-padding-mode'" face))))
+
+(defun spacious-padding-set-face-box-padding (face fallback)
+  "Return appropriate face attributes for FACE with FALLBACK face background."
+  (list :box
+        (list
+         :line-width (spacious-padding--get-face-width face)
+         :color (face-background face nil fallback)
+         :style nil)))
+
 (defun spacious-padding-set-invisible-dividers (_theme)
   "Make window dividers for THEME invisible."
-  (let ((bg (face-background 'default)))
+  (let ((bg-main (face-background 'default))
+        (fg-main (face-foreground 'default)))
     (custom-set-faces
-     `(fringe ((t :background ,bg)))
-     `(window-divider ((t :background ,bg :foreground ,bg)))
-     `(window-divider-first-pixel ((t :background ,bg :foreground ,bg)))
-     `(window-divider-last-pixel ((t :background ,bg :foreground ,bg))))))
+     `(fringe ((t :background ,bg-main)))
+     `(line-number ((t :background ,bg-main)))
+     `(header-line ((t ,@(spacious-padding-set-face-box-padding 'header-line 'default))))
+     `(header-line-highlight ((t :box (:color ,fg-main))))
+     `(mode-line ((t ,@(spacious-padding-set-face-box-padding 'mode-line 'default))))
+     ;; We cannot use :inherit mode-line because it does not get our version of it...
+     `(mode-line-active ((t ,@(spacious-padding-set-face-box-padding 'mode-line-active 'mode-line))))
+     `(mode-line-inactive ((t ,@(spacious-padding-set-face-box-padding 'mode-line-inactive 'mode-line))))
+     `(mode-line-highlight ((t :box (:color ,fg-main))))
+     `(tab-bar-tab ((t ,@(spacious-padding-set-face-box-padding 'tab-bar-tab 'tab-bar))))
+     `(tab-bar-tab-inactive ((t ,@(spacious-padding-set-face-box-padding 'tab-bar-tab-inactive 'tab-bar))))
+     `(window-divider ((t :background ,bg-main :foreground ,bg-main)))
+     `(window-divider-first-pixel ((t :background ,bg-main :foreground ,bg-main)))
+     `(window-divider-last-pixel ((t :background ,bg-main :foreground ,bg-main))))))
 
 (defun spacious-padding-unset-invisible-dividers ()
   "Make window dividers for THEME invisible."
   (custom-set-faces
    '(fringe (( )))
+   '(line-number (( )))
+   '(header-line (( )))
+   '(header-line-highlight (( )))
+   '(mode-line (( )))
+   '(mode-line-active (( )))
+   '(mode-line-inactive (( )))
+   '(mode-line-highlight (( )))
+   '(tab-bar-tab (( )))
+   '(tab-bar-tab-inactive (( )))
    '(window-divider (( )))
    '(window-divider-first-pixel (( )))
    '(window-divider-last-pixel (( )))))
@@ -101,29 +183,22 @@
     (setq spacious-padding--scroll-bar-width
           (frame-parameter nil 'scroll-bar-width))))
 
-(defun spacious-padding--get-internal-border-width (&optional reset)
-  "Return value of frame parameter `internal-border-width'.
+(defmacro spacious-padding--define-get-frame-param (parameter fallback)
+  "Define function to return frame PARAMETER or reset it with FALLBACK value."
+  `(defun ,(intern (format "spacious-padding--get-%s" parameter)) (&optional reset)
+     ,(format "Return value of frame parameter `%s'.
 With optional RESET argument as non-nil, restore the default
 parameter value."
-  (if reset
-      (or spacious-padding--internal-border-width 0)
-    (plist-get spacious-padding-widths :internal-border-width)))
+             parameter)
+     (or
+      (if reset
+          ,(intern (format "spacious-padding--%s" parameter))
+        (plist-get spacious-padding-widths ,(intern (concat ":" parameter))))
+      ,fallback)))
 
-(defun spacious-padding--get-right-divider-width (&optional reset)
-  "Return value of frame parameter `right-divider-width'.
-With optional RESET argument as non-nil, restore the default
-parameter value."
-  (if reset
-      (or spacious-padding--right-divider-width 1)
-    (plist-get spacious-padding-widths :right-divider-width)))
-
-(defun spacious-padding--get-scroll-bar-width (&optional reset)
-  "Return value of frame parameter `scroll-bar-width'.
-With optional RESET argument as non-nil, restore the default
-parameter value."
-  (if reset
-      (or spacious-padding--scroll-bar-width 16)
-    (plist-get spacious-padding-widths :scroll-bar-width)))
+(spacious-padding--define-get-frame-param "internal-border-width" 0)
+(spacious-padding--define-get-frame-param "right-divider-width" 1)
+(spacious-padding--define-get-frame-param "scroll-bar-width" 16)
 
 (defun spacious-padding-modify-frame-parameters (reset)
   "Modify all frame parameters to specify spacing.
@@ -134,14 +209,6 @@ parameter values."
      (right-divider-width . ,(spacious-padding--get-right-divider-width reset))
      (scroll-bar-width  . ,(spacious-padding--get-scroll-bar-width reset)))))
 
-;;;###autoload
-(define-minor-mode spacious-padding-mode
-  "Increase the padding/spacing of frames and windows."
-  :global t
-  (if spacious-padding-mode
-      (spacious-padding--enable-mode)
-    (spacious-padding--disable-mode)))
-
 (defun spacious-padding--enable-mode ()
   "Enable `spacious-padding-mode'."
   (spacious-padding--store-default-parameters)
@@ -155,5 +222,13 @@ parameter values."
   (spacious-padding-unset-invisible-dividers)
   (remove-hook 'enable-theme-functions #'spacious-padding-set-invisible-dividers))
 
+;;;###autoload
+(define-minor-mode spacious-padding-mode
+  "Increase the padding/spacing of frames and windows."
+  :global t
+  (if spacious-padding-mode
+      (spacious-padding--enable-mode)
+    (spacious-padding--disable-mode)))
+
 (provide 'spacious-padding)
 ;;; spacious-padding.el ends here
